@@ -3,6 +3,7 @@ from collections import defaultdict
 from collections.abc import Iterator, Iterable
 from math import ceil
 from datetime import datetime
+import warnings
 
 from django.apps import apps
 from django.db import models
@@ -11,7 +12,7 @@ from django.core.serializers.json import DjangoJSONEncoder
 from django.utils.timezone import make_aware
 
 from datagrowth import settings as datagrowth_settings
-from datagrowth.utils import ibatch, reach
+from datagrowth.utils import ibatch, reach, is_hashable
 from .base import DataStorage
 
 
@@ -35,12 +36,24 @@ class CollectionBase(DataStorage):
         Annotation = apps.get_model("{}.Annotation".format(self._meta.app_label))
         return Annotation.objects.filter(reference__in=self.documents.values("reference"))
 
-    def init_document(self, data, collection=None):
+    def build_document(self, data, collection=None):
+        if hasattr(self, "init_document"):
+            warnings.warn("Collection.init_document method is deprecated in favour of Collection.build_document",
+                          DeprecationWarning)
+            document = self.init_document(data, collection)
+            document.clean()  # this gets handles by Document.build, but not by the legacy Collection.init_document
+            return document
         Document = self.get_document_model()
-        return Document(
-            collection=collection,
-            properties=data
-        )
+        return Document.build(data, collection=collection)
+
+    @property
+    def document_update_fields(self):
+        """
+        Specified which fields on Document are required to update in bulk_update operations
+
+        :return list: field names
+        """
+        return ["properties", "identity", "reference", "modified_at"]
 
     @classmethod
     def validate(cls, data, schema):
@@ -57,18 +70,68 @@ class CollectionBase(DataStorage):
         for instance in data:
             Document.validate(instance, schema)
 
-    def add(self, data, reset=False, batch_size=500, collection=None, modified_at=None, validate=True):
+    def reload_document_ids(self, documents):
         """
-        Add new data to the Collection in batches, possibly deleting all data before adding.
+        Reloads given documents from the database if ``identifier`` is set. Reloaded documents always have an id.
+        Unfortunately MySQL doesn't work well with Django's batch create,
+        because ids won't be set for MySQL when using batch_create.
+        This method is here to provide a workaround for MySQL,
+        because sometimes having ids of new instances is required.
+        Using ``reload_document_ids`` on Documents that have ids does nothing.
 
-        :param data: The data to use for the update
+        :param documents: the documents that may or may not have ids
+        :return: the documents with ids
+        """
+        assert self.identifier, "Can't reload document ids if Collection is not setting Document identity"
+        reloaded = []
+        document_identities = []
+        for document in documents:
+            if document.id:
+                reloaded.append(document)
+            elif document.identity is None:
+                raise ValueError("Can't reload document id if identity is unknown")
+            else:
+                document_identities.append(document.identity)
+        if len(document_identities):
+            Document = self.get_document_model()
+            reloaded += Document.objects.filter(identity__in=document_identities)
+        return reloaded
+
+    def add(self, data, reset=False, collection=None, modified_at=None):
+        """
+        Add new data to the Collection, possibly deleting all data before adding.
+        This method will load all data into memory before inserting it into the database.
+        Use ``add_batches`` for a more memory footprint friendly version.
+
+        :param data: The data to use for the inserts
         :param reset: (optional) whether to delete existing data or not (no by default)
-        :param batch_size: (optional) how many instances to add in a single batch (default: 500)
         :param collection: (optional) a collection instance to add the data to (default: self)
         :param modified_at: (optional) the datetime to use as modified_at value for the collection (default: now)
-        :param validate: (deprecated) used to allow JSON schema validation before addition
-        :return: A list of updated or created instances.
+        :return: A list of created instances.
         """
+        batches = self.add_batches(
+            data,
+            reset=reset,
+            collection=collection,
+            modified_at=modified_at
+        )
+        additions = []
+        for batch in batches:
+            additions += batch
+        return additions
+
+    def add_batches(self, data, batch_size=None, reset=False, collection=None, modified_at=None):
+        """
+        Add new data to the Collection, possibly deleting all data before adding.
+
+        :param data: The data to use for the inserts
+        :param batch_size: The amount of objects to load in memory and insert at once
+        :param reset: (optional) whether to delete existing data or not (no by default)
+        :param collection: (optional) a collection instance to add the data to (default: self)
+        :param modified_at: (optional) the datetime to use as modified_at value for the collection (default: now)
+        :return: An iterator of batches with created instances.
+        """
+        batch_size = batch_size or datagrowth_settings.DATAGROWTH_MAX_BATCH_SIZE
         collection = collection or self
         modified_at = modified_at or make_aware(datetime.now())
         Document = collection.get_document_model()
@@ -78,88 +141,120 @@ class CollectionBase(DataStorage):
         if reset:
             self.documents.all().delete()
 
-        def prepare_additions(data):
+        def prepare_additions(initial_data):
 
             prepared = []
-            if isinstance(data, dict):
-                document = self.init_document(data, collection=collection)
-                document.clean()
+            if isinstance(initial_data, dict):
+                document = self.build_document(initial_data, collection=collection)
+                prepared.append(document)
+            elif isinstance(initial_data, Document):
+                document = self.build_document(initial_data.properties, collection=collection)
                 prepared.append(document)
-            elif isinstance(data, Document):
-                data = self.init_document(data.properties, collection=collection)
-                data.clean()
-                prepared.append(data)
             else:  # type is list
-                for instance in data:
+                for instance in initial_data:
                     prepared += prepare_additions(instance)
             return prepared
 
-        count = 0
+        # Make instances in data unique by hash to prevent adding instances with identical hashes
+        # This is mostly relevant for models that override the __hash__ method,
+        # because Django won't delete instances with identical hashes even when using Document.objects.all().delete().
+        # We can't force uniqueness for Iterator based updates, because that would mean loading all instances in memory.
+        if isinstance(data, list) and len(data):
+            unique_instances = {
+                obj if is_hashable(obj) else ix: obj
+                for ix, obj in enumerate(data)
+            }
+            data = list(unique_instances.values())
+
         for additions in ibatch(data, batch_size=batch_size):
             additions = prepare_additions(additions)
-            count += len(additions)
-            Document.objects.bulk_create(additions, batch_size=datagrowth_settings.DATAGROWTH_MAX_BATCH_SIZE)
+            yield Document.objects.bulk_create(additions, batch_size=batch_size)
 
         if collection.modified_at.replace(microsecond=0) != modified_at.replace(microsecond=0):
             collection.modified_at = modified_at
             collection.save()
-        return count
 
-    def update(self, data, by_property, batch_size=32, collection=None, modified_at=None, validate=True):
+    def update(self, data, by_property, collection=None, modified_at=None):
         """
-        Update data to the Collection in batches, using a property value to identify which Documents to update.
+        Update data to the Collection, using a property value to identify which Documents to update.
+        Any data that does not exist will be added instead.
+        This method will load all data into memory before performing upserts into the database.
+        Use ``update_batches`` for a more memory footprint friendly version.
 
-        :param data: The data to use for the update
+        :param data: The data to use for the upsert
         :param by_property: The property to identify a Document with
-        :param batch_size: (optional) how many instances to add in a single batch (default: 32)
-        :param collection: (optional) a collection instance to update the data to (default: self)
+        :param collection: (optional) a collection instance to upsert the data to (default: self)
         :param modified_at: (optional) the datetime to use as modified_at value for the collection (default: now)
-        :param validate: (deprecated) used to allow JSON schema validation before updates
         :return: A list of updated or created instances.
         """
+        batches = self.update_batches(
+            data,
+            by_property=by_property,
+            collection=collection,
+            modified_at=modified_at
+        )
+        upserts = []
+        for batch in batches:
+            upserts += batch
+        return upserts
+
+    def update_batches(self, data, by_property, batch_size=32, collection=None, modified_at=None):
+        """
+        Update data to the Collection, using a property value to identify which Documents to update.
+        Any data that does not exist will be added instead.
+
+        :param data: The data to use for the upsert
+        :param by_property: The property to identify a Document with
+        :param batch_size: The amount of objects to load in memory and insert at once
+        :param collection: (optional) a collection instance to upsert the data to (default: self)
+        :param modified_at: (optional) the datetime to use as modified_at value for the collection (default: now)
+        :return: An iterator of batches with updated or created instances.
+        """
+        batch_size = batch_size or datagrowth_settings.DATAGROWTH_MAX_BATCH_SIZE
         collection = collection or self
         modified_at = modified_at or make_aware(datetime.now())
         Document = collection.get_document_model()
         assert isinstance(data, (Iterator, list, tuple,)), \
             f"Collection.update expects data to be formatted as iteratable not {type(data)}"
 
-        count = 0
-        for updates in ibatch(data, batch_size=batch_size):
+        for update_data in ibatch(data, batch_size=batch_size):
             # We bulk update by getting all objects whose property matches
             # any update's "by_property" property value and then updating these source objects.
             # One update object can potentially target multiple sources
             # if multiple objects with the same value for the by_property property exist.
-            updated = set()
-            prepared = []
+            exists = set()
+            updates = []
             sources_by_lookup = defaultdict(list)
-            for update in updates:
+            for update in update_data:
+                # If input is a dict we cast it to Document to allow __eq__ methods to do their job.
+                if isinstance(update, dict):
+                    update = self.build_document(update, collection=collection)
                 sources_by_lookup[update[by_property]].append(update)
             target_filters = Q()
             for lookup_value in sources_by_lookup.keys():
                 target_filters |= Q(**{f"properties__{by_property}": lookup_value})
             for target in collection.documents.filter(target_filters):
+                has_update = False
                 for update_value in sources_by_lookup[target.properties[by_property]]:
-                    target.update(update_value, commit=False)
-                count += 1
-                updated.add(target.properties[by_property])
-                prepared.append(target)
-            Document.objects.bulk_update(
-                prepared,
-                ["properties", "identity", "reference", "modified_at"],
-                batch_size=datagrowth_settings.DATAGROWTH_MAX_BATCH_SIZE
-            )
+                    if target != update_value:  # this will be False unless a __eq__ override decides otherwise
+                        has_update = True
+                        target.update(update_value, commit=False)
+                exists.add(target.properties[by_property])
+                if has_update:
+                    updates.append(target)
+            Document.objects.bulk_update(updates, self.document_update_fields, batch_size=batch_size)
             # After all updates we add all data that hasn't been used in any update operation
             additions = []
             for lookup_value, sources in sources_by_lookup.items():
-                if lookup_value not in updated:
+                if lookup_value not in exists:
                     additions += sources
             if len(additions):
-                count += self.add(additions, batch_size=batch_size, collection=collection, modified_at=modified_at)
+                additions = self.add(additions, collection=collection, modified_at=modified_at)
+            yield updates + additions
 
         if collection.modified_at.replace(microsecond=0) != modified_at.replace(microsecond=0):
             collection.modified_at = modified_at
             collection.save()
-        return count
 
     @property
     def content(self):
