@@ -32,10 +32,24 @@ class DocumentBase(DataStorage):
     def __setitem__(self, key, value):
         self.properties[key] = value
 
+    @classmethod
+    def build(cls, data, collection=None):
+        """
+        Build a new Document without committing to the database.
+        Will perform a clean to correctly set attributes based on input data.
+
+        :param data: The data to set as properties of the new Document
+        :param collection: The collection for the new Document
+        :return:
+        """
+        instance = cls(properties=data, collection=collection)
+        instance.clean()
+        return instance
+
     @staticmethod
     def validate(data, schema):
         """
-        Validates the data against given schema and checks validity of ds_id and ds_spirit.
+        Validates the data against given schema
 
         :param data: The data to validate
         :param schema: The JSON schema to use for validation.
@@ -60,16 +74,15 @@ class DocumentBase(DataStorage):
             django_exception.schema = exc.schema
             raise django_exception
 
-    def update(self, data, commit=True, validate=True):
+    def update(self, data, commit=True):
         """
         Update the properties with new data.
 
         :param data: The data to use for the update
         :param commit: Whether to commit new values to the database or not
-        :param validate: (deprecated) used to validate data using JSON schema
         :return: Updated content
         """
-        content = data.content if isinstance(data, DocumentBase) else data
+        content = data.properties if isinstance(data, DocumentBase) else data
         self.properties.update(content)
         self.clean()
         if commit:
@@ -94,23 +107,44 @@ class DocumentBase(DataStorage):
         return self.output_from_content(self.content, *args)
 
     @staticmethod
-    def output_from_content(content, *args):
+    def output_from_content(content, *args, replacement_character="$"):
+        # Assert and abbreviate the replacement_character input
+        assert replacement_character != "\\", "Can't use '\' as a replacement character"
+        rpl = replacement_character
+        # When dealing with multiple args we'll handle it one by one using map function
         if len(args) > 1:
-            return map(DocumentBase.output_from_content, repeat(content), args)
+            mapping = map(
+                lambda cnt, arg: DocumentBase.output_from_content(cnt, arg, replacement_character=rpl),
+                repeat(content), args
+            )
+            return list(mapping)
+        # From here we'll check different types of the input and return values accordingly
         frm = args[0]
         if not frm:
             return frm
-        if isinstance(frm, str):
+        # This is the case that matters most. It replaces a JSON path with a value from the input content
+        if isinstance(frm, str) and frm.startswith(replacement_character):
+            frm = frm.replace(rpl, "$", 1)
             return reach(frm, content)
+        # When dealing with a list as args we'll recursively call this function
+        # Making sure that we'll always return a list
         elif isinstance(frm, list):
             if len(frm) > 1:
-                return DocumentBase.output_from_content(content, *frm)
+                return DocumentBase.output_from_content(content, *frm, replacement_character=rpl)
             else:
-                return [DocumentBase.output_from_content(content, *frm)]
+                return [DocumentBase.output_from_content(content, *frm, replacement_character=rpl)]
+        # When dealing with a dict as args we'll recursively call this function on the values
+        # and make sure we always return a dict
         elif isinstance(frm, dict):
-            return {key: DocumentBase.output_from_content(content, value) for key, value in frm.items()}
-        else:
-            raise AssertionError("Expected a string, list or dict as argument got {} instead".format(type(frm)))
+            return {
+                key: DocumentBase.output_from_content(content, value, replacement_character=rpl)
+                for key, value in frm.items()
+            }
+        # We'll remove any backslash characters from the start of strings
+        elif isinstance(frm, str) and frm.startswith("\\"):
+            frm = frm.replace("\\", "", 1)
+        # Passing along input as-is
+        return frm
 
     def items(self):
         return self.properties.items()
