@@ -1,3 +1,6 @@
+import inspect
+from typing import Union, Callable, Tuple, Dict, Type
+
 from django.apps import apps
 
 from datagrowth.configuration import ConfigurationProperty, ConfigurationType
@@ -8,7 +11,7 @@ class ArgumentsTypes:
     BATCH = 'batch'
 
 
-class Processor(object):
+class Processor:
     """
     This class is the base class for all processors.
     All processors have a config attribute that contains the configuration for the ``Processor``.
@@ -80,3 +83,34 @@ class Processor(object):
 
 class QuerySetProcessor(Processor):
     pass
+
+
+class ProcessorFactory:
+
+    def __init__(self, processor: Union[Type[Processor], str], defaults: Dict = None, method: str = None):
+        self.defaults = defaults or {}
+        if isinstance(processor, str):
+            processor_name, method_name = Processor.get_processor_components(processor)
+            self.processor = Processor.get_processor_class(processor_name)
+            self.method = method_name
+        elif inspect.isclass(processor) and issubclass(processor, Processor):
+            self.processor = processor
+            self.method = method
+        else:
+            raise TypeError("Expected a Processor or name and method of a Processor to build")
+
+    def build(self, config: Union[ConfigurationType, dict] = None, **kwargs) -> Processor:
+        config = config or {}
+        if isinstance(config, ConfigurationType):
+            config.supplement(self.defaults)
+        else:
+            config.update(self.defaults)
+        return self.processor(config)
+
+    def build_with_callable(self, config: Union[ConfigurationType, dict] = None, asynchronous: bool = False,
+                            **kwargs) -> Tuple[Processor, Callable]:
+        prc = self.build(config=config, **kwargs)
+        clb = getattr(prc, self.method) if self.method else prc
+        if asynchronous:
+            clb = getattr(clb, "delay")
+        return prc, clb
