@@ -15,10 +15,9 @@ from bs4 import BeautifulSoup
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils.timezone import now
-try:
-    from django.db.models import JSONField
-except ImportError:
-    from django.contrib.postgres.fields import JSONField
+from django.db.models import JSONField
+from django.test import Client
+from django.urls import reverse
 
 from datagrowth import settings as datagrowth_settings
 from datagrowth.resources.base import Resource
@@ -34,11 +33,13 @@ class HttpResource(Resource):
     This class is a wrapper around the requests library and provides:
 
     * easy follow up of continuation URL's in responses
-    * handle authentication through Datagrowth configs
     * cached responses when retrieving data a second time
+    * handle authentication through setting headers or GET parameters without storing credentials
+    * slowing down requests when servers give errors or warnings related to high-load
 
     Response headers, body and status get stored in the database as well as an abstraction of the request.
     Any authentication data gets stripped before storage in the database.
+    Override handle_errors method to customize how errors in responses are detected.
     """
 
     # Identification
@@ -191,7 +192,7 @@ class HttpResource(Resource):
             elif content_type == "text/html":
                 return content_type, BeautifulSoup(self.body, "html.parser")
             elif content_type == "text/xml" or content_type == "application/xml":
-                return content_type, BeautifulSoup(self.body, "lxml")
+                return content_type, BeautifulSoup(self.body, "xml")
             else:
                 return content_type, None
         return None, None
@@ -297,7 +298,7 @@ class HttpResource(Resource):
         assert isinstance(request, dict), "Request should be a dictionary."
         method = request.get("method")
         assert method, "Method should not be falsy."
-        assert method in ["get", "post", "put"], \
+        assert method in ["get", "post", "put", "head"], \
             "{} is not a supported resource method.".format(request.get("method"))  # FEATURE: allow all methods
         if validate_input:
             self._validate_input(
@@ -489,7 +490,8 @@ class HttpResource(Resource):
                     preq,
                     proxies=datagrowth_settings.DATAGROWTH_REQUESTS_PROXIES,
                     verify=datagrowth_settings.DATAGROWTH_REQUESTS_VERIFY,
-                    timeout=self.timeout
+                    timeout=self.timeout,
+                    allow_redirects=self.config.allow_redirects
                 )
                 self._update_from_results(response)
             except requests.exceptions.SSLError:
@@ -505,7 +507,6 @@ class HttpResource(Resource):
             if self.status not in [420, 429, 502, 503, 504]:
                 break
 
-
     def _update_from_results(self, response):
         self.head = dict(response.headers.lower_items())
         self.status = response.status_code
@@ -630,7 +631,7 @@ class HttpResource(Resource):
         :return: mime_type, encoding
         """
         match = re.match(
-            "(?P<mime_type>[A-Za-z]+/[A-Za-z]+);? ?(charset=(?P<encoding>[A-Za-z0-9\-]+))?",
+            r"(?P<mime_type>[A-Za-z]+/[A-Za-z]+);? ?(charset=(?P<encoding>[A-Za-z0-9\-]+))?",
             content_type
         )
         if match is None:
@@ -735,3 +736,52 @@ class MicroServiceResource(HttpResource):
 
     class Meta:
         abstract = True
+
+
+class TestClientResource(HttpResource):
+    """
+    You can extend from this base class to declare a ``Resource`` that gathers data through Django's test Client.
+    By setting ``test_view_name`` to a named Django urlpattern this ``Resource`` will fetch data from the view,
+    associated with that urlpattern.
+
+    This ``Resource ``can be useful to create tests for Datagrowth components,
+    where data comes from Django views that output test data.
+    """
+
+    test_view_name = None
+
+    def _create_url(self, *args):
+        variables = self.variables(*args)
+        path = reverse(self.test_view_name, args=variables["url"])
+        params = self.parameters(**variables)
+        return f"{path}?{urlencode(params, doseq=True)}"
+
+    def _send(self):
+        assert self.request and isinstance(self.request, dict), \
+            "Trying to make request before having a valid request dictionary."
+
+        method = self.request.get("method")
+        form_data = self.request.get("data") if not method == "get" else None
+        json_data = self.request.get("json") if not method == "get" else None
+        data = json_data or form_data or None
+
+        client = Client()
+        client_method = getattr(client, method)
+        path = self.request["url"]
+        if data:
+            content_type = "application/octet-stream" if not json_data else "application/json"
+            response = client_method(path, data=data, content_type=content_type, follow=True)
+        else:
+            response = client_method(path, follow=True)
+        self._update_from_results(response)
+
+    def _update_from_results(self, response):
+        self.head = {
+            key: value[1]  # extracts the lower case headers
+            for key, value in response.headers._store.items()
+        }
+        self.status = response.status_code
+        self.body = response.content.decode("utf-8", "replace")
+
+    class Meta:
+        abstract = True
