@@ -1,5 +1,4 @@
 import logging
-from time import sleep
 
 from django.apps import apps
 from celery import current_app as app
@@ -8,7 +7,7 @@ from datagrowth.configuration import load_config
 from datagrowth.exceptions import DGResourceException
 
 
-log = logging.getLogger("datascope")
+log = logging.getLogger("datagrowth")
 
 
 @app.task(name="shell_resource.run")
@@ -25,7 +24,7 @@ def run(config, *args, **kwargs):
         cmd.close()
         success.append(cmd.id)
     except DGResourceException as exc:
-        log.debug(exc)
+        log.log(config.resource_exception_log_level, exc)
         cmd = exc.resource
         cmd.close()
         errors.append(cmd.id)
