@@ -1,3 +1,4 @@
+import logging
 import os
 
 from django.conf import settings
@@ -17,14 +18,15 @@ DATAGROWTH_DATETIME_FORMAT = getattr(settings, "DATAGROWTH_DATETIME_FORMAT", "%Y
 
 DATAGROWTH_DATA_DIR = getattr(settings, "DATAGROWTH_DATA_DIR", os.path.join(settings.BASE_DIR, "data"))
 DATAGROWTH_MEDIA_ROOT = getattr(settings, "DATAGROWTH_MEDIA_ROOT", settings.MEDIA_ROOT)
-DATAGROWTH_BIN_DIR = getattr(settings, "DATAGROWTH_BIN_DIR",
+DATAGROWTH_BIN_DIR = getattr(
+    settings, "DATAGROWTH_BIN_DIR",
     os.path.join(settings.BASE_DIR, "datagrowth", "resources", "shell", "bin")
 )
 
 DATAGROWTH_REQUESTS_PROXIES = getattr(settings, "DATAGROWTH_REQUESTS_PROXIES", None)
 DATAGROWTH_REQUESTS_VERIFY = getattr(settings, "DATAGROWTH_REQUESTS_VERIFY", True)
 
-DATAGROWTH_MAX_BATCH_SIZE = getattr(settings, "DATAGROWTH_MAX_BATCH_SIZE", 500)
+DATAGROWTH_MAX_BATCH_SIZE = getattr(settings, "DATAGROWTH_MAX_BATCH_SIZE", 100)
 
 DATAGROWTH_KALDI_BASE_PATH = getattr(settings, "DATAGROWTH_KALDI_BASE_PATH", "")
 DATAGROWTH_KALDI_ASPIRE_BASE_PATH = getattr(settings, "DATAGROWTH_KALDI_ASPIRE_BASE_PATH", "")
@@ -37,18 +39,37 @@ DATAGROWTH_KALDI_NL_BASE_PATH = getattr(settings, "DATAGROWTH_KALDI_NL_BASE_PATH
 
 
 DATAGROWTH_DEFAULT_CONFIGURATION = getattr(settings, "DATAGROWTH_DEFAULT_CONFIGURATION", {
+    # Global configurations that control multiple classes
     "global_asynchronous": True,  # by default offload to celery where possible
-    "global_async": True,  # legacy "asynchronous" configuration for Python <= 3.6
+    "global_sample_size": 0,
+    "global_datatypes_app_label": None,
+    "global_datatype_models": {
+        "document": "Document",
+        "collection": "Collection",
+        "dataset_version": "DatasetVersion",
+        "process_result": "ProcessResult",
+        "batch": "Batch"
+    },
+    # Legacy global pipeline configuration, use datatype configurations instead
+    "global_pipeline_app_label": None,
+    "global_pipeline_models": {
+        "document": "Document",
+        "process_result": "ProcessResult",
+        "batch": "Batch"
+    },
+    # Resources specific "global" configurations
     "global_user_agent": "DataGrowth (v{})".format(DATAGROWTH_VERSION),
     "global_purge_after": {},
     "global_purge_immediately": False,  # by default keep resources around
-    "global_sample_size": 0,
     "global_cache_only": False,
+    "global_resource_exception_log_level": logging.DEBUG,
 
     "http_resource_continuation_limit": 1,
     "http_resource_interval_duration": 0,  # NB: milliseconds!
     "http_resource_concat_args_size": 0,
     "http_resource_concat_args_symbol": "|",
+    # TODO: these two configurations should be http_resource, not global
+    "global_allow_redirects": True,
     "global_backoff_delays": [2, 4, 8, 16],
 
     "shell_resource_interval_duration": 0,  # NB: milliseconds!
@@ -62,9 +83,6 @@ DATAGROWTH_DEFAULT_CONFIGURATION = getattr(settings, "DATAGROWTH_DEFAULT_CONFIGU
     "google_api_key": getattr(settings, 'GOOGLE_API_KEY', ''),
     "google_cx": getattr(settings, 'GOOGLE_CX', ''),
 
-    "indico_api_key": getattr(settings, 'INDICO_API_KEY', ''),
-    "wizenoze_api_key": getattr(settings, 'WIZENOZE_API_KEY', ''),
-
     "rank_processor_batch_size": 1000,
     "rank_processor_result_size": 20,
 
